#!/usr/bin/env python3

from nmigen.lib.cdc import ResetSynchronizer
#from nmigen.build.dsl import *
from nmigen import (Elaboratable, Signal, Module, DomainRenamer,
                    ClockDomain, Instance)

class PLL(Elaboratable):
    def __init__(self, clk125):
        self.clk_125 = clk125
        self.clk_40 = Signal()
        self.pll_locked = Signal()
    def elaborate(self, platform):
        m=Module()

        pll_fb = Signal()
        pll_40 = Signal()
        m.submodules += [
            Instance("PLLE2_BASE",
                p_STARTUP_WAIT="FALSE", o_LOCKED=self.pll_locked,
                    
                # VCO @ 1GHz
                p_REF_JITTER1=0.01, p_CLKIN1_PERIOD=8.0,
                p_CLKFBOUT_MULT=8, p_DIVCLK_DIVIDE=1,
                i_CLKIN1=self.clk_125, i_CLKFBIN=pll_fb, o_CLKFBOUT=pll_fb,
                # 40MHz
                p_CLKOUT1_DIVIDE=25, p_CLKOUT1_PHASE=0.0, o_CLKOUT1=pll_40,
            ),
            Instance("BUFG", i_I=pll_40, o_O=self.clk_40),
        ]
        return m

class VGA_test(Elaboratable):
    def elaborate(self, platform):
        m=Module()
        m.submodules.vgen = vgen = DomainRenamer("cd40")(VGA_signal_gen())

        m.submodules.clkgen = clkgen = PLL(platform.request("clk125"))

        cd40 = ClockDomain("cd40")
        cd_sync = ClockDomain()

        m.domains += cd40, cd_sync

        m.d.comb += cd40.clk.eq(clkgen.clk_40)
        m.submodules += ResetSynchronizer(~clkgen.pll_locked,domain="cd40")
        return m

    def ports(self):
        return []


class VGA_signal_gen(Elaboratable):
    def elaborate(self, platform):
        m = Module()
        m.d.sync += Signal().eq(1)
        return m

from nmigen_boards.arty_z7 import ArtyZ720Platform
ArtyZ720Platform().build(VGA_test(), do_program=True)
